@@ -1,16 +1,17 @@
 # -*- coding: utf-8 -*-
 """
-    Library providing all necessary tools for PDF interpolation.
+Library providing all necessary tools for PDF interpolation.
 
-    This library also provides a class to generate the interpolator `InterpolatorDispatcher`.
-    Upon construction the dispatcher generates a number of functions
-    to evaluate the interpolator.
+This library also provides a class to generate the interpolator :class:`InterpolatorDispatcher`.
+Upon construction the dispatcher generates a number of functions
+to evaluate the interpolator.
 """
 import logging
 import math
 
-import numpy as np
 import numba as nb
+import numpy as np
+import scipy.special as sp
 
 logger = logging.getLogger(__name__)
 
@@ -50,13 +51,13 @@ class Area:
         self.coefs = self._compute_coefs(xgrid)
 
     def _reference_indices(self):
-        """ Iterate over all indices which are part of the block """
+        """Iterate over all indices which are part of the block"""
         for k in range(self.kmin, self.kmax + 1):
             if k != self.poly_number:
                 yield k
 
     def _compute_coefs(self, xgrid):
-        """ Compute the coefficients for this area given a grid on x """
+        """Compute the coefficients for this area given a grid on x"""
         denominator = 1.0
         coeffs = np.ones(1)
         xj = xgrid[self.poly_number]
@@ -70,7 +71,7 @@ class Area:
         return coeffs
 
     def __iter__(self):
-        """ Iterates the generated coefficients """
+        """Iterates the generated coefficients"""
         for coef in self.coefs:
             yield coef
 
@@ -175,6 +176,10 @@ def evaluate_Nx(N, logx, area_list):
     return res
 
 
+# TODO lift to runcard?
+_atol_eps = 10 * np.finfo(float).eps
+
+
 @nb.njit("f8(f8,f8[:,:])", cache=True)
 def evaluate_x(x, area_list):
     """
@@ -200,7 +205,7 @@ def evaluate_x(x, area_list):
         xmin = a[0]
         xmax = a[1]
         coefs = a[2:]
-        if xmin < x <= xmax or (j == 0 and x == xmin):
+        if xmin < x <= xmax or (j == 0 and np.abs(x - xmin) < _atol_eps):
             for i, coef in enumerate(coefs):
                 res += coef * pow(x, i)
             return res
@@ -238,7 +243,7 @@ class BasisFunction:
     defined by (xmin-xmax) and containing a list of coefficients.
 
     Upon construction will generate all areas and generate and compile
-    a function to evaluate in N (or x) the iterpolator
+    a function to evaluate in N (or x) the interpolator
 
     Parameters
     ----------
@@ -314,7 +319,7 @@ class BasisFunction:
                 area config
         """
         # This is necessary as numba will ask for everything
-        # to be inmutable
+        # to be immutable
         area_list = []
         for area in self:
             area_list.append([area.xmin, area.xmax, *area.coefs])
@@ -357,7 +362,7 @@ class BasisFunction:
         r"""
         Compiles the function to evaluate the interpolator in N space.
 
-        Generates a function `evaluate_Nx` with a (N, logx) signature.
+        Generates a function :meth:`evaluate_Nx` with a (N, logx) signature.
 
         .. math::
             \tilde p(N)*\exp(- N * \ln(x))
@@ -385,8 +390,8 @@ class InterpolatorDispatcher:
     """
     Setups the interpolator.
 
-    Upon construction will generate a list of `BasisFunction` objects.
-    Each of these `BasisFunction` objects exponses a `callable`
+    Upon construction will generate a list of :class:`BasisFunction` objects.
+    Each of these :class:`BasisFunction` objects expose a `callable`
     method (also accessible as the `__call__` method of the class)
     which will be numba-compiled.
 
@@ -433,9 +438,12 @@ class InterpolatorDispatcher:
         self.xgrid = xgrid
         self.polynomial_degree = polynomial_degree
         self.log = log
-        logger.info("Interpolation: number of points = %d", xgrid_size)
-        logger.info("Interpolation: polynomial degree = %d", polynomial_degree)
-        logger.info("Interpolation: logarithmic = %s", log)
+        logger.info(
+            "Interpolation: number of points = %d, polynomial degree = %d, logarithmic = %s",
+            xgrid_size,
+            polynomial_degree,
+            log,
+        )
 
         # Create blocks
         list_of_blocks = []
@@ -496,8 +504,12 @@ class InterpolatorDispatcher:
         """
         # load xgrid
         xgrid = operators_card["interpolation_xgrid"]
+        if len(xgrid) == 0:
+            raise ValueError("Empty xgrid!")
         if xgrid[0] == "make_grid":
             xgrid = make_grid(*xgrid[1:])
+        elif xgrid[0] == "make_lambert_grid":
+            xgrid = make_lambert_grid(*xgrid[1:])
         is_log_interpolation = bool(operators_card["interpolation_is_log"])
         polynom_rank = operators_card["interpolation_polynomial_degree"]
         return cls(
@@ -550,10 +562,10 @@ class InterpolatorDispatcher:
         # compute map
         out = []
         for x in targetgrid:
-            l = []
+            row = []
             for b in self.basis:
-                l.append(b.evaluate_x(x))
-            out.append(l)
+                row.append(b.evaluate_x(x))
+            out.append(row)
         return np.array(out)
 
     def to_dict(self):
@@ -590,7 +602,7 @@ def make_grid(
         n_low : int
             points in the small-x region
         n_mid : int
-            points in the mediun-x region
+            points in the medium-x region
         n_high : int
             points in the large-x region
         x_min : float
@@ -620,3 +632,54 @@ def make_grid(
     # join
     xgrid = np.unique(np.concatenate((xgrid_low, xgrid_mid, xgrid_high, np.array([1]))))
     return xgrid
+
+
+def make_lambert_grid(
+    n_pts,
+    x_min=1e-7,
+    x_max=1.0,
+):
+    r"""
+    Creates a smoothly spaced grid that is linear near 1 and logarithmic near 0.
+
+    It is generated by the relation:
+
+    .. math::
+        x(y) = \frac{1}{5} W_{0}(5 \exp(5-y))
+
+    where :math:`W_{0}` is the principle branch of the
+    :func:`Lambert W function <scipy.special.lambertw>` and
+    :math:`y` is a variable which extremes are given as function of :math:`x`
+    by the direct relation:
+
+    .. math::
+        y(x) = 5(1-x)-\log(x)
+
+    This method is implemted in `PineAPPL`, :cite:`Carrazza_2020` eq 2.11 and relative
+    paragraph.
+
+    Parameters
+    ----------
+        n_pts : int
+            number of points
+        x_min : float
+            minimum x (included)
+        x_max : float
+            maximum x (included)
+
+    Returns
+    -------
+        xgrid : numpy.ndarray
+            generated grid
+    """
+
+    def direct_relation(x):
+        return 5 * (1 - x) - np.log(x)
+
+    def inverse_relation(y):
+        return np.real(1 / 5 * sp.lambertw(5 * np.exp(5 - y)))
+
+    y_min = direct_relation(x_min)
+    y_max = direct_relation(x_max)
+
+    return np.array([inverse_relation(y) for y in np.linspace(y_min, y_max, n_pts)])
